@import './variables';
@import 'vuetify/styles/main.sass';
@import './override';
@import './layout/container';
@import './layout/sidebar';

@import './components/VButtons';
@import './components/VCard';
@import './components/VField';
@import './components/VInput';
@import './components/VNavigationDrawer';
@import './components/VShadow';
@import './components/VTextField';
@import './components/VTabs';

@import './pages/dashboards';
