from astrbot.core.star.register import (
    register_star as register,  # 注册插件（Star）
)

from astrbot.core.star import Context, Star, StarTools
from astrbot.core.star.config import *

__all__ = ["register", "Context", "Star", "StarTools"]
