.v-input--density-default,
.v-field--variant-solo,
.v-field--variant-filled {
  --v-input-control-height: 51px;
  --v-input-padding-top: 14px;
}
.v-input--density-comfortable {
  --v-input-control-height: 56px;
  --v-input-padding-top: 17px;
}
.v-label {
  font-size: 0.975rem;
}
.v-switch .v-label,
.v-checkbox .v-label {
  opacity: 1;
}
